React = require 'react'
PropTypes = require 'prop-types'
createClass = require 'create-react-class'
ReactDOM = require 'react-dom'
map = require 'lodash/map'
isEqual = require 'lodash/isEqual'

MultiSelectValue = require './multi_select_value'
MultiSelectOptionList = require './multi_select_option_list'

{makeGuid} = require '../utils'
{ESCAPE, DOWN_ARROW, SPACE} = require '../constants/keyboard'

{div, button} = require 'react-dom-factories'

###
Multi Select Props

@props.options - REQUIRED - Array
  An array of options for the user to click - can be a flat array (where each entry is both the value and label)
  or an array of objects, where the value is the `valueField` and the label is the `labelField` (see below)

@props.values - OPTIONAL - Array
  A flat array of values that the control initializes to. If the options array is objects, each entry should be the 
  `valueField` value. Will default to an empty array

@props.labelField - OPTIONAL - String
  The attribute name from each option object that is the user facing label

@props.valueField - OPTIONAL - String
  The attribute name from each option object that is the 'value`

@props.returnFullObjects - OPTIONAL - Boolean - default: false
  Whether or not the values returned from the input are a collection of objects or a flat array of values

@props.onChange - OPTIONAL - function
  Function/method to fire when the data changes

@props.placeholder - OPTIONAL
  placeholder text for an empty control

@props.tabIndex - OPTIONAL
  tab order of the edi button

@props.onRemove - OPTIONAL - function
  function call when an item is removed, will pass the removed item
###

MultiSelect = createClass
  
  displayName: 'MultiSelect'

  propTypes:
    values: PropTypes.array
    options: PropTypes.array
    jsonPath: PropTypes.string
    returnFullObjects: PropTypes.bool
    disabled: PropTypes.bool
    isInPopover: PropTypes.bool
    labelField: PropTypes.string
    valueField: PropTypes.string
    tabIndex: PropTypes.number
    placeholder: PropTypes.string
    closeOnSelect: PropTypes.bool

  contextTypes:
    clearValidationError: PropTypes.func
    addValidationError: PropTypes.func
    getValidationStatus: PropTypes.func
    toggleValidationError: PropTypes.func
    openOverlay: PropTypes.func
    closeOverlay: PropTypes.func
    
  getDefaultProps: ->
    {
      values: []
      options: []
      returnFullObjects: yes
      isInPopover: no 
      disabled: no
      labelField: 'label'
      valueField: 'value'
      defaultPlaceholder: 'Select from list...'
      closeOnSelect: no
    }

  getInitialState: ->
    {values} = @props  
    {
      valueHasChanged: no
      values: values.slice(0)
      focused: no
    }

  componentDidMount: -> 
    {validation} = @props
    {values} = @state
    
    # Create a unique id for this input
    # Used by the validation framework to group validation errors
    @inputId = makeGuid()
    
    @validate validation, values

    @focusBtn.addEventListener 'focus', @handleFocus
    @focusBtn.addEventListener 'blur', @handleBlur

  componentWillReceiveProps: (nextProps) ->
    {values, validation, valueField, returnFullObjects} = nextProps

    validationChanged = (
      if typeof validation is 'function' then no
      else not isEqual(validation?.messages, @props.validation?.messages)
    )

    # Run validaiton if the validation changes
    if validationChanged then @validate(validation, values) 
    
    # This will update the state should the value change externally after the component is mounted
    # This is an ugly side effect of transferring props to state, but also needing to react to prop changes
    # Try a quick compare of length
    if @state.values.length isnt values.length
      @setState {values} 
      @validate validation, values
    # Otherwise resort to 
    else
      for value, i in @state.values
        if returnFullObjects
          break if value[valueField] isnt values[i]?[valueField] 
        else 
          break if value isnt values[i]
      
      # Don't update state if no differnces were found
      return if i is @state.values.length
      
      @setState {values} 
      @validate validation, values
  
  componentWillUnmount: -> 
    @context.clearValidationError 
      groupId: @inputId
      isInPopover: @props.isInPopover
    
    @focusBtn.removeEventListener 'focus', @handleFocus
    @focusBtn.removeEventListener 'blur', @handleBlur

  render: ->
    {valueHasChanged, focused, values} = @state
    {options, tabIndex, disabled, valueField, labelField, tabIndex, placeholder, defaultPlaceholder} = @props
    {getValidationStatus} = @context
    {error, forceShowAllErrors} = getValidationStatus(@inputId)
    
    isValid = not error?

    className = 'multiselect field-wrap' 
    className += ' invalid' if not isValid and (valueHasChanged or forceShowAllErrors)
    className += ' disabled' if disabled
    className += ' focused' if focused

    selected = (MultiSelectValue {
      key: value[valueField] or value
      value
      labelField
      disabled
      @onRemove
    } for value in values)

    if selected.length is 0
      selected = div {
        key: 'placeholder'
        className: 'placeholder'
      }, if placeholder? then placeholder else (if t? then t(defaultPlaceholder) else defaultPlaceholder)

    div {
      className
      onClick: if not disabled then @handleClick else null
      ref: (@anchor) =>
    }, [
      button {
        key: 'focusBtn'
        className: 'focus-btn'
        ref: (@focusBtn) =>
        onKeyDown: @handleKeydown
        tabIndex
        disabled
      } 
      selected
      div {
        className: 'field-errors-show'
        key: 'textInputErrorsShow'
        ref: 'errorAnchor'
        onMouseOver: @handleErrorMouseOver
        onMouseOut: @handleErrorMouseOut
      }
    ]

  getValue: -> @state.values.slice(0)

  handleClick: ->
    return if @pvrId?
    {focused} = @state

    @focusBtn.focus() if not focused

    do @renderOptionList
    
  renderOptionList: ->
    {openOverlay} = @context
    {options, labelField, valueField, returnFullObjects, closeOnSelect} = @props
    {values} = @state

    @pvrId = openOverlay
      component: MultiSelectOptionList
      id: 'popover'
      noBackdrop: true
      disableBackgroundControls: no
      overlayId: @pvrId
      props: {
        @anchor
        options
        values
        @onRemove
        @onAdd
        labelField
        valueField
        returnFullObjects
        closeOnSelect
        onClose: @handlePvrClose
      }


  handleKeydown: ({keyCode}) ->
    return if keyCode not in [SPACE, DOWN_ARROW] or @pvrId?

    do @handleClick

  handleChange: ->
    {jsonPath, onChange, validation, closeOnSelect} = @props
    {closeOverlay} = @context
    values = @getValue()
    onChange?(values, jsonPath)
    @validate validation, values
    if closeOnSelect
      closeOverlay(@pvrId) if @pvrId?
    else
      do @renderOptionList if @pvrId?

  onAdd: (newValue) ->
    {values} = @state
    {returnFullObjects, valueField} = @props
    newValue = if returnFullObjects then newValue else newValue[valueField]
    values.push newValue
    valueHasChanged = yes

    @setState {values, valueHasChanged}, @handleChange

  onRemove: (removedValue) ->
    {values} = @state
    {valueField, returnFullObjects} = @props

    for item, i in values when (if returnFullObjects then item[valueField] is removedValue[valueField] else item is removedValue[valueField])
      values.splice(i, 1)
      break
    
    valueHasChanged = yes

    @setState {values, valueHasChanged}, @handleChange

  handlePvrClose: -> 
    # Don't delete the pvr Id until the next event loop
    # This will prevent a space bar keydown event event from closing AND then immediately reopening the drop down
    setTimeout =>
      delete @pvrId
    , 0

  handleFocus: -> 
    @setState {focused: yes}
    #document.addEventListener 'keydown', @handleKeydown

  handleBlur: -> 
    @setState {focused: no}
    #document.addEventListener 'keydown', @handleKeydown

  validate: (validation, value) ->
    return if validation is off 

    # Run validation and show any auto show messages
    if typeof validation is 'function' then validationError = validation(value)
    # validation can also be passed as a static array
    else validationError = validation

    {isInPopover, tabId} = @props
    {addValidationError, clearValidationError} = @context

    if validationError?
      addValidationError
        anchor: @refs.errorAnchor
        error: validationError
        groupId: @inputId
        isInPopover: isInPopover
        tabId: tabId
    else 
      clearValidationError
        groupId: @inputId
        isInPopover: isInPopover
  
  handleErrorMouseOver: ->
    {isInPopover} = @props
    @context.toggleValidationError 
      groupId: @inputId
      status: on
      isInPopover: isInPopover

  handleErrorMouseOut: ->
    {isInPopover} = @props
    @context.toggleValidationError
      groupId: @inputId
      status: off
      isInPopover: isInPopover


module.exports = MultiSelect